/*****************************************************************************
             LATTICE GENERATOR FOR POV-Ray 3.1
                   (include file 2)

 Created by: Thomas de Groot (the Netherlands)
     e-mail: t.degroot@inter.nl.net
         On: 20 January 1998
    Revised: 29 September 1998
    Updated: 01 May 1999
   Contains: "SquareLattice", "CubeLattice", "FlatLattice", "SpaceLattice" and "CageLattice"
  Copyright: May be freely used and modified. Please acknowledge author.
 *****************************************************************************

General info needed for the use of self-defined pipes:
 using Pipe1 (10x2x2): Translation should be = 17.5
 using Pipe2 (15x2x2): Translation should be = 22.5
 using Pipe3 (20x2x2): Translation should be = 27.5
 using Pipe4 (25x2x2): Translation should be = 32.5
 using Pipe5 (30x2x2): Translation should be = 37.5
 using Pipe6 (35x2x2): Translation should be = 42.5
 using Pipe7 (40x2x2): Translation should be = 47.5
 etc...
 */
//=============================================================
#ifndef (X)
 #declare X = 1;
#end
#ifndef (Y)
 #declare Y = 1;
#end
#ifndef (Z)
 #declare Z = 1;
#end
#ifndef (RowIteration)
 #declare RowIteration = 4;
#end
#ifndef (ColIteration)
 #declare ColIteration = 4;
#end
#ifndef (LevIteration)
 #declare LevIteration = 4;
#end

#declare RowIteration = RowIteration - 1;
#declare ColIteration = ColIteration - 1;
#declare LevIteration = LevIteration - 1;
//=============================================================
#switch (X)
 #case (1)
  #declare XPipe = object { Pipe1 }
  #declare XTranslation = 17.5;
 #break
 #case (2)
  #declare XPipe = object { Pipe2 }
  #declare XTranslation = 22.5;
 #break
 #case (3)
  #declare XPipe = object { Pipe3 }
  #declare XTranslation = 27.5;
 #break
 #case (4)
  #declare XPipe = object { Pipe4 }
  #declare XTranslation = 32.5;
 #break
 #case (5)
  #declare XPipe = object { Pipe5 }
  #declare XTranslation = 37.5;
 #break
 #case(6)
  #declare XPipe = object { Pipe6 }
  #declare XTranslation = 42.5;
 #break
 #case (7)
  #declare XPipe = object { Pipe7 }
  #declare XTranslation = 47.5;
#end
//=============================================================
#switch (Y)
 #case (1)
  #declare YPipe = object { Pipe1 }
  #declare YTranslation = 17.5;
 #break
 #case (2)
  #declare YPipe = object { Pipe2 }
  #declare YTranslation = 22.5;
 #break
 #case (3)
  #declare YPipe = object { Pipe3 }
  #declare YTranslation = 27.5;
 #break
 #case (4)
  #declare YPipe = object { Pipe4 }
  #declare YTranslation = 32.5;
 #break
 #case (5)
  #declare YPipe = object { Pipe5 }
  #declare YTranslation = 37.5;
 #break
 #case (6)
  #declare YPipe = object { Pipe6 }
  #declare YTranslation = 42.5;
 #break
 #case (7)
  #declare YPipe = object { Pipe7 }
  #declare YTranslation = 47.5;
#end
//=============================================================
#switch (Z)
 #case (1)
  #declare ZPipe = object { Pipe1 }
  #declare ZTranslation = 17.5;
 #break
 #case (2)
  #declare ZPipe = object { Pipe2 }
  #declare ZTranslation = 22.5;
 #break
 #case (3)
  #declare ZPipe = object { Pipe3 }
  #declare ZTranslation = 27.5;
 #break
 #case (4)
  #declare ZPipe = object { Pipe4 }
  #declare ZTranslation = 32.5;
 #break
 #case (5)
  #declare ZPipe = object { Pipe5 }
  #declare ZTranslation = 37.5;
 #break
 #case (6)
  #declare ZPipe = object { Pipe6 }
  #declare ZTranslation = 42.5;
 #break
 #case (7)
  #declare ZPipe = object { Pipe7 }
  #declare ZTranslation = 47.5;
#end
//=============================================================
//======================= DECLARATIONS ========================
//=============================================================
#declare RowMax = (ZTranslation*RowIteration)+ZTranslation;
#declare RowMin =  ZTranslation*RowIteration;

#declare ColMax = (XTranslation*ColIteration)+XTranslation;
#declare ColMin =  XTranslation*ColIteration;

#declare LevMax = (YTranslation*LevIteration)+YTranslation;
#declare LevMin =  YTranslation*LevIteration;

#declare XShift = XTranslation/2;
#declare YShift = YTranslation/2;
#declare ZShift = ZTranslation/2;

//=============================================================
//======================= FLAT LATTICE ========================
//=============================================================
#declare FlatLattice =
union {
 //PIPE LATTICE:
 #declare Row = 0;
 #declare Column = 0;
 #while (Row <= RowMax)
  #while (Column <= ColMin)
   object { XPipe
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #while (Row <= RowMin)
  #while (Column <= ColMax)
   object { ZPipe
    rotate <0, 90, 0>
    translate <-XShift, 0, ZShift>
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //INNER JOINTS:
 #declare Row = ZTranslation;
 #declare Column = XShift;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   object { FourFlatPipes
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XShift;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //RIBS:
 #declare Row = ZTranslation;
 #declare Column = -XShift;
 #while (Row <= RowMin)
  object { ThreeFlatPipes
   rotate <0, 90, 0>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #while (Row <= RowMin)
  object { ThreeFlatPipes
   rotate <0, -90, 0>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { ThreeFlatPipes
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { ThreeFlatPipes
   rotate <0, 180, 0>
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 //-------------------------------------------------------------
 //CORNERS:
 object { TwoFlatPipes
  rotate <0, 180, 0>
  translate <-XShift, 0, 0>
 }

 object { TwoFlatPipes
  rotate <0, -90, 0>
  translate <-XShift, 0, RowMax>
 }

 object { TwoFlatPipes
  rotate <0, 90, 0>
  translate <ColMin+XShift, 0, 0>
 }

 object { TwoFlatPipes
  translate <ColMin+XShift, 0, RowMax>
 }
}

//=============================================================
//==================== (3-D) SPACE LATTICE ====================
//=============================================================
#declare SpaceLattice =
union {
 //PIPE LATTICE:
 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row <= RowMax)
  #while (Column <= ColMin)
   #while (Level <= LevMax)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row <= RowMin)
  #while (Column <= ColMax)
   #while (Level <= LevMax)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row <= RowMax)
  #while (Column <= ColMax)
   #while (Level <= LevMin)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //INNER JOINTS:
 #declare Row = ZTranslation;
 #declare Column = XShift;
 #declare Level = YTranslation;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   #while (Level <= LevMin)
    object { SixPipes
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level =YTranslation;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XShift;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //RIBS:
 #declare Row = ZTranslation;         // Base
 #declare Column = -XShift;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <0, 90, 90>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <90, -90, 180>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 0, 180>
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 180, 180>
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end
//-------------------------------------------------------
 #declare Row = ZTranslation;
 #declare Column = -XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <90, 90, 90>
   translate <Column, Level, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <0, 90, -90>
   translate <Column, Level, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = XShift;
 #declare Level = LevMax;
 #while (Column <= ColMin)
  object { FourRibPipes
   translate <Column, Level, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = XShift;
 #declare Level = LevMax;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 180, 0>
   translate <Column, Level, Row>
  }
  #declare Column = Column+XTranslation;
 #end
//-------------------------------------------------------
 #declare Row = 0;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <0, 0, 90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = 0;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <0, 0, -90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <90, 0, 90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <90, 0, -90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 //-------------------------------------------------------------
 //--------------------------------------> SIDES: Base
 #declare Row = ZTranslation;
 #declare Column = XTranslation-XShift;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   object { FivePipes
    rotate <-90, 0, 0>
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XTranslation-XShift;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Top
 #declare Row = ZTranslation;
 #declare Column = XTranslation-XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   object { FivePipes
    rotate <90, 0, 0>
    translate <Column, Level, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XTranslation-XShift;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Front
 #declare Row = 0;
 #declare Column = XShift;
 #declare Level = YTranslation;
 #while (Column <= ColMin)
  #while (Level <= LevMin)
   object { FivePipes
    rotate <0, 0, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Column = Column+XTranslation;
 #end

 //--------------------------------------> SIDES: Back
 #declare Row = RowMax;
 #declare Column = XShift;
 #declare Level = YTranslation;
 #while (Column <= ColMin)
  #while (Level <= LevMin)
   object { FivePipes
    rotate <180, 0, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Column = Column+XTranslation;
 #end

 //--------------------------------------> SIDES: Left
 #declare Row = ZTranslation;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Row <= RowMin)
  #while (Level <= LevMin)
   object { FivePipes
    rotate <0, 90, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Right
 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Row <= RowMin)
  #while (Level<= LevMin)
   object { FivePipes
    rotate <0, -90, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //CORNERS:
 object { ThreeCornerPipes
  rotate <0, 180, 90>
  translate <-XShift, 0, 0>
 }

 object { ThreeCornerPipes
  rotate <0, -90, 90>
  translate <-XShift, 0, RowMax>
 }

 object { ThreeCornerPipes
  rotate <180, 0, 0>
  translate <ColMin+XShift, 0, 0>
 }

 object { ThreeCornerPipes
  rotate <90, 0, 0>
  translate <ColMin+XShift, 0, RowMax>
 }
 object { ThreeCornerPipes
  rotate <0, 180, 0>
  translate <-XShift, LevMax, 0>
 }

 object { ThreeCornerPipes
  rotate <0, -90, 0>
  translate <-XShift, LevMax, RowMax>
 }

 object { ThreeCornerPipes
  rotate <0, 90, 0>
  translate <ColMin+XShift, LevMax, 0>
 }

 object { ThreeCornerPipes
  translate <ColMin+XShift, LevMax, RowMax>
 }
}

//=============================================================
//=================== HOLLOW SPACE LATTICE ====================
//=============================================================
#declare CageLattice =
union {
 //PIPE LATTICE:
 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 //--------------------------------------> ROWS: Front
  #while (Column <= ColMin)
   #while (Level <= LevMax)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = RowMax;

 //--------------------------------------> ROWS: Back
  #while (Column <= ColMin)
   #while (Level <= LevMax)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = 0;
  #declare Row = Row+ZTranslation;

 //--------------------------------------> ROWS: Bottom
  #while (Row < RowMax)
   #while (Column < ColMax)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Column = Column+XTranslation;
   #end
   #declare Column = 0;
   #declare Row = Row+ZTranslation;
  #end
  #declare Column = 0;
  #declare Row = 0;
  #declare Row = Row+ZTranslation;
  #declare Level = LevMax;

 //--------------------------------------> ROWS: Top
  #while (Row < RowMax)
   #while (Column < ColMax)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Column = Column+XTranslation;
   #end
   #declare Column = 0;
   #declare Row = Row+ZTranslation;
  #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 //--------------------------------------> COLUMNS: Left
  #while (Row < RowMax)
   #while (Level <= LevMax)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Row = Row+ZTranslation;
  #end
  #declare Row = 0;
  #declare Column = ColMax;

 //--------------------------------------> COLUMNS: Right
  #while (Row < RowMax)
   #while (Level <= LevMax)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Row = Row+ZTranslation;
  #end
  #declare Row = 0;
  #declare Column = 0;
  #declare Column = Column+XTranslation;

 //--------------------------------------> COLUMNS: Bottom
  #while (Row < RowMax)
   #while (Column < ColMax)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Column = Column+XTranslation;
   #end
   #declare Column = 0;
   #declare Row = Row+ZTranslation;
  #end
  #declare Row = 0;
  #declare Column = 0;
  #declare Column = Column+XTranslation;
  #declare Level = LevMax;

 //--------------------------------------> COLUMNS: Top
  #while (Row < RowMax)
   #while (Column < ColMax)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Column = Column+XTranslation;
   #end
   #declare Column = 0;
   #declare Row = Row+ZTranslation;
  #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 //--------------------------------------> LEVELS: Front
  #while (Column <= ColMax)
   #while (Level < LevMax)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = RowMax;

 //--------------------------------------> LEVELS: Back
  #while (Column <= ColMax)
   #while (Level < LevMax)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = 0;
  #declare Row = Row+ZTranslation;

 //--------------------------------------> LEVELS: Left
  #while (Row <= RowMax)
   #while (Level < LevMax)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Row = Row+ZTranslation;
  #end
  #declare Column = ColMax;
  #declare Row = 0;
  #declare Row = Row+ZTranslation;

 //--------------------------------------> LEVELS: Right
  #while (Row <= RowMax)
   #while (Level < LevMax)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Row = Row+ZTranslation;
  #end

 //-------------------------------------------------------------
 //RIBS:
 #declare Row = ZTranslation;         // Base
 #declare Column = -XShift;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <0, 90, 90>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <90, -90, 180>
   translate <Column, 0, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 0, 180>
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = XShift;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 180, 180>
   translate <Column, 0, Row>
  }
  #declare Column = Column+XTranslation;
 #end
//-------------------------------------------------------
 #declare Row = ZTranslation;
 #declare Column = -XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <90, 90, 90>
   translate <Column, Level, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  object { FourRibPipes
   rotate <0, 90, -90>
   translate <Column, Level, Row>
  }
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = XShift;
 #declare Level = LevMax;
 #while (Column <= ColMin)
  object { FourRibPipes
   translate <Column, Level, Row>
  }
  #declare Column = Column+XTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = XShift;
 #declare Level = LevMax;
 #while (Column <= ColMin)
  object { FourRibPipes
   rotate <0, 180, 0>
   translate <Column, Level, Row>
  }
  #declare Column = Column+XTranslation;
 #end
//-------------------------------------------------------
 #declare Row = 0;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <0, 0, 90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = 0;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <0, 0, -90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <90, 0, 90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 #declare Row = RowMax;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Level <= LevMin)
  object { FourRibPipes
   rotate <90, 0, -90>
   translate <Column, Level, Row>
  }
  #declare Level = Level+YTranslation;
 #end

 //-------------------------------------------------------------
 //--------------------------------------> SIDES: Base
 #declare Row = ZTranslation;
 #declare Column = XTranslation-XShift;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   object { FourFlatPipes
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XTranslation-XShift;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Top
 #declare Row = ZTranslation;
 #declare Column = XTranslation-XShift;
 #declare Level = LevMax;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   object { FourFlatPipes
    translate <Column, Level, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = XTranslation-XShift;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Front
 #declare Row = 0;
 #declare Column = XShift;
 #declare Level = YTranslation;
 #while (Column <= ColMin)
  #while (Level <= LevMin)
   object { FourFlatPipes
    rotate <90, 0, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Column = Column+XTranslation;
 #end

 //--------------------------------------> SIDES: Back
 #declare Row = RowMax;
 #declare Column = XShift;
 #declare Level = YTranslation;
 #while (Column <= ColMin)
  #while (Level <= LevMin)
   object { FourFlatPipes
    rotate <90, 0, 0>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Column = Column+XTranslation;
 #end

 //--------------------------------------> SIDES: Left
 #declare Row = ZTranslation;
 #declare Column = -XShift;
 #declare Level = YTranslation;
 #while (Row <= RowMin)
  #while (Level <= LevMin)
   object { FourFlatPipes
    rotate <0, 0, 90>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Row = Row+ZTranslation;
 #end

 //--------------------------------------> SIDES: Right
 #declare Row = ZTranslation;
 #declare Column = ColMin+XShift;
 #declare Level = YTranslation;
 #while (Row <= RowMin)
  #while (Level<= LevMin)
   object { FourFlatPipes
    rotate <0, 0, 90>
    translate <Column, Level, Row>
   }
   #declare Level = Level+YTranslation;
  #end
  #declare Level = YTranslation;
  #declare Row = Row+ZTranslation;
 #end

//-------------------------------------------------------------
 //CORNERS:
 object { ThreeCornerPipes
  rotate <0, 180, 90>
  translate <-XShift, 0, 0>
 }
 object { ThreeCornerPipes
  rotate <0, -90, 90>
  translate <-XShift, 0, RowMax>
 }
 object { ThreeCornerPipes
  rotate <180, 0, 0>
  translate <ColMin+XShift, 0, 0>
 }
 object { ThreeCornerPipes
  rotate <90, 0, 0>
  translate <ColMin+XShift, 0, RowMax>
 }

 object { ThreeCornerPipes
  rotate <0, 180, 0>
  translate <-XShift, LevMax, 0>
 }
 object { ThreeCornerPipes
  rotate <0, -90, 0>
  translate <-XShift, LevMax, RowMax>
 }
 object { ThreeCornerPipes
  rotate <0, 90, 0>
  translate <ColMin+XShift, LevMax, 0>
 }
 object { ThreeCornerPipes
  translate <ColMin+XShift, LevMax, RowMax>
 }
}

//=============================================================
//====================== SQUARE LATTICE =======================
//=============================================================
#declare SquareLattice =
union {
 #declare RowMin = ZTranslation;
 #declare ColMin = XTranslation;
 //PIPE LATTICE:
 #declare Row = 0;
 #declare Column = 0;
 #while (Row <= RowMin)
  #while (Column < ColMin)
   object { XPipe
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #while (Row < RowMin)
  #while (Column <= ColMin)
   object { ZPipe
    rotate <0, 90, 0>
    translate <-XShift, 0, ZShift>
    translate <Column, 0, Row>
   }
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 //-------------------------------------------------------------
 //CORNERS:
 object { TwoFlatPipes
  rotate <0, 180, 0>
  translate <-XShift, 0, 0>
 }

 object { TwoFlatPipes
  rotate <0, -90, 0>
  translate <-XShift, 0, RowMin>
 }

 object { TwoFlatPipes
  rotate <0, 90, 0>
  translate <XShift, 0, 0>
 }

 object { TwoFlatPipes
  translate <XShift, 0, RowMin>
 }
}

//=============================================================
//==================== (3-D) CUBE LATTICE =====================
//=============================================================
#declare CubeLattice =
union {
 #declare RowMin = ZTranslation;
 #declare ColMin = XTranslation;
 #declare LevMin = YTranslation;
 //PIPE LATTICE:
 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row <= RowMin)
  #while (Column < ColMin)
   #while (Level <= LevMin)
    object { XPipe
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row < RowMin)
  #while (Column <= ColMin)
   #while (Level <= LevMin)
    object { ZPipe
     rotate <0, 90, 0>
     translate <-XShift, 0, ZShift>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end

 #declare Row = 0;
 #declare Column = 0;
 #declare Level = 0;
 #while (Row <= RowMin)
  #while (Column <= ColMin)
   #while (Level < LevMin)
    object { YPipe
     rotate <0, 0, 90>
     translate <-XShift, YShift, 0>
     translate <Column, Level, Row>
    }
    #declare Level = Level+YTranslation;
   #end
   #declare Level = 0;
   #declare Column = Column+XTranslation;
  #end
  #declare Column = 0;
  #declare Row = Row+ZTranslation;
 #end
 //-------------------------------------------------------------
 //CORNERS:
 object { ThreeCornerPipes
  rotate <0, 180, 90>
  translate <-XShift, 0, 0>
 }

 object { ThreeCornerPipes
  rotate <0, -90, 90>
  translate <-XShift, 0, RowMin>
 }

 object { ThreeCornerPipes
  rotate <180, 0, 0>
  translate <XShift, 0, 0>
 }

 object { ThreeCornerPipes
  rotate <90, 0, 0>
  translate <XShift, 0, RowMin>
 }
 object { ThreeCornerPipes
  rotate <0, 180, 0>
  translate <-XShift, LevMin, 0>
 }

 object { ThreeCornerPipes
  rotate <0, -90, 0>
  translate <-XShift, LevMin, RowMin>
 }

 object { ThreeCornerPipes
  rotate <0, 90, 0>
  translate <XShift, LevMin, 0>
 }

 object { ThreeCornerPipes
  translate <XShift, LevMin, RowMin>
 }
}
